@echo off

rem *** path ***
set work_path=.
set config_file=%work_path%\config_for_gpdata.ini
set tool_path=%work_path%\tools
set gpdata_exe=%tool_path%\gpdata.exe

rem *** config.ini parameter ***
set set_psa_file_name=none
set set_verify_method=none
set set_voltage=none
set set_buzz=none
set set_target_model=none
set set_user_program_address=none
set set_initial_serial_number=none
set set_serial_number_start_address=none
set set_security_password=none
set set_checksum=none

set option_verify_method=
set option_voltage=
set option_buzz=
set option_target_model=
set option_user_program_address=
set option_initial_serial_number=
set option_serial_number_start_address=
set option_security_password=
set option_checksum=

rem *** error ***
set error_delete_gpdata=0
set error_psa_file=0
set error=0

setlocal enabledelayedexpansion


rem *** read config.ini parameter ***
if /i exist %config_file% (
   for /f "tokens=1,2 delims==" %%a in (%config_file%) do (
      if "%%a" == "PSA_FILE_NAME" (
         if not "%%b" == "" (
            set set_psa_file_name=%%b
         )
      )
      if "%%a" == "VERIFICATION_METHOD" (
         if not "%%b" == "" (
            set set_verify_method=%%b
         )
      )
      if "%%a" == "INTERFACE_VOLTAGE" (
         if not "%%b" == "" (
            set set_voltage=%%b
         )
      )
      if "%%a" == "BUZZER" (
         if not "%%b" == "" (
            set set_buzz=%%b
         )
      )
      if "%%a" == "MODEL" (
         if not "%%b" == "" (
            set set_target_model=%%b
         )
      )
      if "%%a" == "USER_PROGRAM_ADDRESS" (
         if not "%%b" == "" (
            set set_user_program_address=%%b
         )
      )
      if "%%a" == "INITIAL_SERIAL_NUMBER" (
         if not "%%b" == "" (
            set set_initial_serial_number=%%b
         )
      )
      if "%%a" == "SERIAL_NUMBER_START_ADDRESS" (
         if not "%%b" == "" (
            set set_serial_number_start_address=%%b
         )
      )
      if "%%a" == "SECURITY_PASSWORD" (
         if not "%%b" == "" (
            set set_security_password=%%b
         )
      )
      if "%%a" == "CHECK_USER_PROGRAM_CHECKSUM" (
         if not "%%b" == "" (
            set set_checksum=%%b
         )
      )

   )
)


rem ******************************************************************************************
rem * Initial display
rem ******************************************************************************************

echo ************************************************************************
echo * Checking configulation file :%config_file%
echo * 



rem ******************************************************************************************
rem * check checksum operation
rem ******************************************************************************************
if %set_checksum% == YES (
  goto :_checksum_
)


rem ******************************************************************************************
rem * delete old gpdata.bin ***
rem ******************************************************************************************

if %set_checksum% == NO (
  del gpdata.bin >NUL 2>&1
  if exist gpdata.bin (
    echo *  ERROR cannot delete old gpdata.bin
    set error_delete_gpdata=1
  )
)



rem ******************************************************************************************
rem
rem * Check psa file
rem ******************************************************************************************

if not %set_psa_file_name:~-4% == .psa (
    echo *  ERROR -psa file is NOT specified.
    set error=1
) else (
   if not exist %set_psa_file_name% (
    echo *  ERROR -specifed psa file does NOT exit.
    set error_psa_file=1
   ) else (
    echo *  OK     PSA_FILE_NAME                 :%set_psa_file_name%
    del %set_psa_file_name:~0,-4%.bin >NUL 2>&1
    %tool_path%\objcopy -I srec -O binary %set_psa_file_name% %set_psa_file_name:~0,-4%.bin
   )
)


rem ******************************************************************************************
rem * check VERIFICATION_METHOD 
rem ******************************************************************************************
if %set_verify_method% == 0 (
    echo *  OK     VERIFICATION_METHOD           :%set_verify_method% ...Checksum verification
    set option_verify_method=-v%set_verify_method%
) else if %set_verify_method% == 1 (
    echo *  OK     VERIFICATION_METHOD           :%set_verify_method% ...All data verification
    set option_verify_method=-v%set_verify_method%
) else if %set_verify_method% == none (
    set error=1
    echo *  ERROR -VERIFICATION_METHOD should be "0,1". Nothing is set.
) else (
    set error=1
    echo *  ERROR -VERIFICATION_METHOD should be "0,1". "%set_verify_method%" is set.
)

rem ******************************************************************************************
rem * check INTERFACE_VOLTAGE
rem ******************************************************************************************
if %set_voltage% == 0 (
    echo *  OK     INTERFACE_VOLTAGE             :%set_voltage% ...External voltage
    set option_voltage=-d%set_voltage%
) else if %set_voltage% == 1 (
    echo *  OK     INTERFACE_VOLTAGE             :%set_voltage% ...1.8 V
    set option_voltage=-d%set_voltage%
) else if %set_voltage% == 2 (
    echo *  OK     INTERFACE_VOLTAGE             :%set_voltage% ...3.3 V
    set option_voltage=-d%set_voltage%
) else if %set_voltage% == none (
    set error=1
    echo *  ERROR -INTERFACE_VOLTAGE should be "0,1,2". Nothing is set.
) else (
    set error=1
    echo *  ERROR -INTERFACE_VOLTAGE should be "0,1,2". "%set_voltage%" is set.
)

rem ******************************************************************************************
rem *check BUZZER
rem ******************************************************************************************
if %set_buzz% == 0 (
    echo *  OK     BUZZER                        :%set_buzz% ...Buzzer Off
    set option_buzz=-b%set_buzz%
) else if %set_buzz% == 1 (
    echo *  OK     BUZZER                        :%set_buzz% ...Buzzer On
    set option_buzz=-b%set_buzz%
) else if %set_buzz% == none (
    set error=1
    echo *  ERROR -BUZZER should be "0,1". Nothing is set.
) else (
    set error=1
    echo *  ERROR -BUZZER should be "0,1". "%set_buzz%" is set.
)

rem ******************************************************************************************
rem * check MODEL
rem ******************************************************************************************
call :_count_length %set_target_model%

set error_target_model=FALSE 
if %length% GTR 10 set error_target_model=TRUE 

if %set_target_model% == none (
    set error=1
    echo *  ERROR -MODEL should be set. Nothing is set.
) else if %error_target_model% == TRUE (
    set error=1
    echo *  ERROR -MODEL should be less than 10 characters. "%set_target_model%" is set.
) else (
    echo *  OK     MODEL                         :%set_target_model%
    set option_target_model=-t%set_target_model%
) 

rem ******************************************************************************************
rem * check USER_PROGRAM_ADDRESS
rem ******************************************************************************************
if %set_user_program_address:~0,2%  == 0x (
  call :_count_length %set_user_program_address:~2%
  call :_check_hex %set_user_program_address:~2%
  set limit_digit=8
) else (
  call :_count_length %set_user_program_address%
  call :_check_dec %set_user_program_address%
  set limit_digit=10
)

set error_user_program_address=FALSE 
if %length% GTR %limit_digit% set error_user_program_address=TRUE 
if %length% EQU 0 set error_user_program_address=TRUE 

if %set_user_program_address% == none (
    set error=1
    echo *  ERROR -USER_PROGRAM_ADDRESS should be set 0x00000000 to 0xFFFFFFFF. Nothing is set.
) else if not "%input%"=="" (
  set error=1
  echo *  ERROR -USER_PROGRAM_ADDRESS :"%set_user_program_address%" is set. including character "%input%" 
) else if %error_user_program_address% == TRUE (
    set error=1
    echo *  ERROR -USER_PROGRAM_ADDRESS should be set 0x00000000 to 0xFFFFFFFF. "%set_user_program_address%" is set.
) else (
    echo *  OK     USER_PROGRAM_ADDRESS          :%set_user_program_address%
    set option_user_program_address=-a%set_user_program_address%
) 

rem ******************************************************************************************
rem * check INITIAL_SERIAL_NUMBER & SERIAL_NUMBER_START_ADDRESS
rem ******************************************************************************************
if %set_initial_serial_number% == NO (
   if not %set_serial_number_start_address% == NO (
     set error=1
     echo *  ERROR -Both INITIAL_SERIAL_NUMBER and SERIAL_NUMBER_START_ADDRESS should be set
     goto _password_
   )
) else if not %set_initial_serial_number% == NO (
   if %set_serial_number_start_address% == NO (
    set error=1
     echo *  ERROR -Both INITIAL_SERIAL_NUMBER and SERIAL_NUMBER_START_ADDRESS should be set
     goto _password_
   )
)

rem ******************************************************************************************
rem * check INITIAL_SERIAL_NUMBER
rem ******************************************************************************************
if %set_initial_serial_number:~0,2%  == 0x (
  call :_count_length %set_initial_serial_number:~2%
  call :_check_hex %set_initial_serial_number:~2%
  set limit_digit=8
) else (
  call :_count_length %set_initial_serial_number%
  call :_check_dec %set_initial_serial_number%
  set limit_digit=10
)

set error_initial_serial_number=FALSE 
if %length% GTR %limit_digit% set error_initial_serial_number=TRUE 
if %length% EQU 0 set error_initial_serial_number=TRUE 

if %set_initial_serial_number% == none (
    set error=1
    echo *  ERROR -INITIAL_SERIAL_NUMBER should be set 0x00000000 to 0xFFFFFFFF. Nothing is set.
) else if %set_initial_serial_number% == NO (
    echo *  OK     INITIAL_SERIAL_NUMBER         :%set_initial_serial_number% ...this option is NOT set.
) else if not "%input%"=="" (
    set error=1
    echo *  ERROR -INITIAL_SERIAL_NUMBER :"%set_initial_serial_number%" is set. including character "%input%" 
) else if %error_initial_serial_number% == TRUE (
    set error=1
    echo *  ERROR -INITIAL_SERIAL_NUMBER should be set 0x00000000 to 0xFFFFFFFF. "%set_initial_serial_number%" is set.
) else (
    echo *  OK     INITIAL_SERIAL_NUMBER         :%set_initial_serial_number%
    set option_initial_serial_number=-i%set_initial_serial_number%
) 

rem ******************************************************************************************
rem * check SERIAL_NUMBER_START_ADDRESS
rem ******************************************************************************************
if %set_serial_number_start_address:~0,2%  == 0x (
  call :_count_length %set_serial_number_start_address:~2%
  call :_check_hex %set_serial_number_start_address:~2%
  set limit_digit=8
) else (
  call :_count_length %set_serial_number_start_address%
  call :_check_dec %set_serial_number_start_address%
  set limit_digit=10
)

set error_serial_number_start_address=FALSE 
if %length% GTR %limit_digit% set error_serial_number_start_address=TRUE 
if %length% EQU 0 set error_serial_number_start_address=TRUE 

if %set_serial_number_start_address% == none (
    set error=1
    echo *  ERROR -SERIAL_NUMBER_START_ADDRESS should be set 0x00000000 to 0xFFFFFFFF. Nothing is set.
) else if %set_serial_number_start_address% == NO (
    echo *  OK     SERIAL_NUMBER_START_ADDRESS   :%set_serial_number_start_address% ...this option is NOT set
) else if not "%input%"=="" (
    set error=1
    echo *  ERROR -SERIAL_NUMBER_START_ADDRESS :"%set_serial_number_start_address%" is set. including character "%input%" 
) else if %error_serial_number_start_address% == TRUE (
    set error=1
    echo *  ERROR -SERIAL_NUMBER_START_ADDRESS should be set 0x00000000 to 0xFFFFFFFF. "%set_serial_number_start_address%" is set.
) else (
    echo *  OK     SERIAL_NUMBER_START_ADDRESS   :%set_serial_number_start_address%
    set option_serial_number_start_address=-s%set_serial_number_start_address%
) 

rem ******************************************************************************************
rem * check SECURITY_PASSWORD
rem ******************************************************************************************
:_password_

call :_check_alphanumeric %set_security_password%
call :_count_length %set_security_password%

set error_security_password=FALSE 
if %length% GTR 12 set error_security_password=TRUE 


if %set_security_password% == none (
    set error=1
    echo *  ERROR -SECURITY_PASSWORD should be set "NO" or Max12 alphanumeric characters. Nothing is set.
) else if %set_security_password% == NO (
    echo *  OK     SECURITY_PASSWORD             :%set_security_password% ...this option is NOT set.
) else if not "%input%"=="" (
    set error=1
    echo *  ERROR -SECURITY_PASSWORD :"%set_security_password%" is set. including character "%input%" 
) else if %error_security_password% == TRUE (
    set error=1
    echo *  ERROR -SECURITY_PASSWORD should be set "NO" or Max12 alphanumeric characters
    echo           %length%characters "%set_security_password%" is set.
) else (
    echo *  OK     SERIAL_NUMBER_START_ADDRESS   :%set_security_password%
    set option_security_password=-p%set_security_password%
) 

rem ******************************************************************************************
rem * check CHECK_USER_PROGRAM_CHECKSUM
rem ******************************************************************************************
:_checksum_
if %set_checksum% == YES (
    echo *  OK     CHECK_USER_PROGRAM_CHECKSUM   :%set_checksum% ...display User program checksum
) else if %set_checksum% == NO (
    echo *  OK     CHECK_USER_PROGRAM_CHECKSUM   :%set_checksum% 
) else if %set_checksum% == none (
    set error=1
    echo *  ERROR -CHECK_USER_PROGRAM_CHECKSUM should be "YES,NO". Nothing is set.
) else (
    set error=1
    echo *  ERROR -CHECK_USER_PROGRAM_CHECKSUM should be "YES,NO". "%set_checksum%" is set.
)


rem ******************************************************************************************
rem * ERROR Check
rem ******************************************************************************************
if %error_delete_gpdata% == 1 (
    echo *
    del %set_psa_file_name:~0,-4%.bin >NUL 2>&1
) else if %error_psa_file% == 1 (
    echo *
    del %set_psa_file_name:~0,-4%.bin >NUL 2>&1
) else if %error% == 1 (
    echo *
    del %set_psa_file_name:~0,-4%.bin >NUL 2>&1
)

if %error_delete_gpdata% == 1 (
    echo *  ERROR cannot delete old gpdata.bin
)

if %error_psa_file% == 1 (
    echo *  ERROR -specifed psa file does NOT exit.
)

if %error% == 1 (
    echo *  ERROR in %config_file% 
)

if %error_delete_gpdata% == 1 (
    echo *  NO gpdata.bin is generated!
    echo ************************************************************************
    goto _end_
) else if %error_psa_file% == 1 (
    echo *  NO gpdata.bin is generated!
    echo ************************************************************************
    goto _end_
) else if %error% == 1 (
    echo *  NO gpdata.bin is generated!
    echo ************************************************************************
    goto _end_
)




rem ******************************************************************************************
rem * Create gpdata.bin
rem ******************************************************************************************

echo ************************************************************************
echo.

if %set_checksum% == YES (
   echo ************************************************************************
   echo * Display existing gpdata.bin cheksum value

   if not exist gpdata.bin (
   echo *
   echo *  Warning    No gpdata.bin --- cannot be displayed
   echo *
   echo ************************************************************************
   goto _end_
   )

   echo *  %gpdata_exe% -c
   echo *
   %gpdata_exe% -c
   echo *
   echo ************************************************************************
) else if %set_checksum% == NO (
   echo ************************************************************************
   echo * START Create gpdata.bin
   echo *  %gpdata_exe% %set_psa_file_name:~0,-4%.bin %option_verify_method% %option_voltage% %option_buzz% %option_target_model% %option_user_program_address% %option_initial_serial_number% %option_serial_number_start_address% %option_security_password% 
   %gpdata_exe% %set_psa_file_name:~0,-4%.bin %option_verify_method% %option_voltage% %option_buzz% %option_target_model% %option_user_program_address% %option_initial_serial_number% %option_serial_number_start_address% %option_security_password% >NUL 2>&1
    del %set_psa_file_name:~0,-4%.bin >NUL 2>&1

    if exist gpdata.bin (
       echo *
       echo *  OK     gpdata.bin file is created.
       echo *
       echo ************************************************************************
    ) else (
       echo *
       echo *  ERROR  gpdata.bin file is NOT created.
       echo *
       echo ************************************************************************
    )

)

goto _end_

:_check_hex
  set input=%1
  if not "%input%"=="" set input=%input:0=%
  if not "%input%"=="" set input=%input:1=%
  if not "%input%"=="" set input=%input:2=%
  if not "%input%"=="" set input=%input:3=%
  if not "%input%"=="" set input=%input:4=%
  if not "%input%"=="" set input=%input:5=%
  if not "%input%"=="" set input=%input:6=%
  if not "%input%"=="" set input=%input:7=%
  if not "%input%"=="" set input=%input:8=%
  if not "%input%"=="" set input=%input:9=%
  if not "%input%"=="" set input=%input:A=%
  if not "%input%"=="" set input=%input:B=%
  if not "%input%"=="" set input=%input:C=%
  if not "%input%"=="" set input=%input:D=%
  if not "%input%"=="" set input=%input:E=%
  if not "%input%"=="" set input=%input:F=%
  goto :EOF

:_check_dec
  set input=%1
  if not "%input%"=="" set input=%input:0=%
  if not "%input%"=="" set input=%input:1=%
  if not "%input%"=="" set input=%input:2=%
  if not "%input%"=="" set input=%input:3=%
  if not "%input%"=="" set input=%input:4=%
  if not "%input%"=="" set input=%input:5=%
  if not "%input%"=="" set input=%input:6=%
  if not "%input%"=="" set input=%input:7=%
  if not "%input%"=="" set input=%input:8=%
  if not "%input%"=="" set input=%input:9=%
  goto :EOF


:_count_length
set str=%1
set length=0
:_loop_count_length
if not "%str%"=="" (
    set str=%str:~1%
    set /a length=%length%+1
    goto :_loop_count_length
)
goto :EOF


:_check_alphanumeric
  set input=%1
  if not "%input%"=="" set input=%input:0=%
  if not "%input%"=="" set input=%input:1=%
  if not "%input%"=="" set input=%input:2=%
  if not "%input%"=="" set input=%input:3=%
  if not "%input%"=="" set input=%input:4=%
  if not "%input%"=="" set input=%input:5=%
  if not "%input%"=="" set input=%input:6=%
  if not "%input%"=="" set input=%input:7=%
  if not "%input%"=="" set input=%input:8=%
  if not "%input%"=="" set input=%input:9=%
  if not "%input%"=="" set input=%input:A=%
  if not "%input%"=="" set input=%input:B=%
  if not "%input%"=="" set input=%input:C=%
  if not "%input%"=="" set input=%input:D=%
  if not "%input%"=="" set input=%input:E=%
  if not "%input%"=="" set input=%input:F=%
  if not "%input%"=="" set input=%input:G=%
  if not "%input%"=="" set input=%input:H=%
  if not "%input%"=="" set input=%input:I=%
  if not "%input%"=="" set input=%input:J=%
  if not "%input%"=="" set input=%input:K=%
  if not "%input%"=="" set input=%input:L=%
  if not "%input%"=="" set input=%input:M=%
  if not "%input%"=="" set input=%input:N=%
  if not "%input%"=="" set input=%input:O=%
  if not "%input%"=="" set input=%input:P=%
  if not "%input%"=="" set input=%input:Q=%
  if not "%input%"=="" set input=%input:R=%
  if not "%input%"=="" set input=%input:S=%
  if not "%input%"=="" set input=%input:T=%
  if not "%input%"=="" set input=%input:U=%
  if not "%input%"=="" set input=%input:V=%
  if not "%input%"=="" set input=%input:W=%
  if not "%input%"=="" set input=%input:X=%
  if not "%input%"=="" set input=%input:Y=%
  if not "%input%"=="" set input=%input:Z=%
  goto :EOF


:_end_

echo.

pause
